<?php

namespace App\Models\Service;

use App\Models\DomainRegistrar;
use App\Traits\ServiceDomains;

/**
 * Class Domain (Service)
 * Services that are managed Domain Names
 */
class Domain extends Type
{
	use ServiceDomains;

	protected $table = 'service_domain';
	protected $with = ['tld'];

	/* OVERRIDES */

	/**
	 * Service update validation
	 *
	 * @return array
	 */
	public function validation(): array
	{
		return [
			'domain_name' => 'nullable|string|min:2',
			'tld_id' => 'required|exists:tlds,id',
			'expire_at' => 'required|date',
			'domain_registrar_id' => 'required|exists:domain_registrars,id',
			'registrar_account' => 'nullable|string',
			'registrar_ns' => 'nullable|string',
			'registrar_username' => 'nullable|string',
		];
	}

	/* RELATIONS */

	public function registrar()
	{
		return $this->belongsTo(DomainRegistrar::class,'domain_registrar_id');
	}
}